clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates PCA based motion synthesis. The synthesis can
% be controlled by adjusting the parameters of the linear combination of
% the PCA components. Original input motions can be reproduced by using
% rows of the coefficient matrix that PCA calculation provides as the
% parameters. The example explores effects of PCA components by creating a
% series of motions starting from an original motion and offsetting one
% parameter to positive and negative directions. Offsetting PCA parameters
% can create similar unnatural looking artifacts as N-way extrapolation
% (ex_06_extrapolation_nway.m).

% Each PCA parameter seems to have a different affect on the resulting
% motion. The first parameter allows making changes to the step size
% during a motion. The second parameter can turn a motion to a faster and
% slightly limping version. However, attaching labels that describe styles
% to the PCA components can be difficult and subjective.

%% PCA calculation

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

motions_in_collection = length(walk_motions_a);
quaternion_channels = size(walk_motions_a(1).quaternions, 2);
frames_after_timewarp = size(walk_motions_a(1).warped_bvh, 1);

orig_motions = zeros((quaternion_channels+4)*frames_after_timewarp, motions_in_collection);
% Note the +4 is for 3 root coordinates channels and 1 channel for timewarp info

for i1 = 1 : motions_in_collection
    orig_motions(:, i1) = reshape( ...
        [ walk_motions_a(i1).warped_bvh(:, 1:3) ...
        [0; log(((walk_motions_a(i1).time_transfer(2:end) - walk_motions_a(i1).time_transfer(1:end-1)))')] ...
        walk_motions_a(i1).warped_quaternions  ], (quaternion_channels+4)*frames_after_timewarp, 1);
end

mean_values = mean(orig_motions'); 
orig_motions = orig_motions - repmat(mean_values', 1, motions_in_collection);

[coeffs, ~, latent_synth] = princomp(orig_motions);
% Note: 'princomp' will be replaced by 'pca' in future Matlab versions

% Scores given by 'princomp' have column means subtracted which is not
% necessary in our case, thus we recalculate the scores.
pca_scores = orig_motions * coeffs;

%% Values that can be changed to explore effects of PCA parameters

index_parameter_to_vary = 1;
offsets_to_parameter = [0.75 0.45 0.15 0 -0.15 -0.45 -0.75];
index_of_starting_motion = 1;

%% Synthesis by linear combination of PCA components

count = 0;
for param_change = offsets_to_parameter
    count = count + 1;
    
    parameters = coeffs(index_of_starting_motion, :)';
    parameters(index_parameter_to_vary) = parameters(index_parameter_to_vary) + param_change;
    
    bvhdata{count} = f_parametric_component_synthesis(pca_scores, parameters, mean_values, walk_motions_a(index_of_starting_motion));
    coordinates{count} = f_fast_bvh_to_coordinates(bvhdata{count}(1:end, :), walk_motions_a(index_of_starting_motion).skel);
end
    
%% Visualization

for i1 = 1:length(coordinates)
    f_animate(bvhdata{i1}, walk_motions_a(1).skel, coordinates{i1}, 'figNum', 101);
    title(['PCA synthesis: offset of ' num2str(offsets_to_parameter(i1)) ' to parameter ' num2str(index_parameter_to_vary)]);
    pause(1.0);
end
