clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');
addpath('./external_libraries/FastICA_25');

%% Description of the example

% This example demonstrates ICA based motion synthesis. The synthesis can
% be controlled by adjusting the parameters of the linear combination of
% the ICA components. Motions close to the original input motions can be
% created by using rows of the mixing coefficient matrix that ICA
% calculation provides as the parameters. The example explores effects of
% ICA components by creating a series of motions starting from an original
% motion and offsetting one parameter to positive and negative directions.

% Note that due to the random nature of the ICA, the components are
% different each time they are calculated. It is also possible that the
% calculation fails to converge. In ICA, the dimensionality is reduced by
% calculating PCA components and preserving only part of them. A small
% number PCA components tends to give synthesis results that are similar to
% ones in PCA based synthesis (ex_07_pca_synthesis). A large number of PCA
% components can results in some of the ICA components to look very random.
% Also, some ICA components may require orders of magnitude greater offsets
% than others to produce any perceivable changes to a motion. 

%% ICA calculation

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

motions_in_collection = length(walk_motions_a);
quaternion_channels = size(walk_motions_a(1).quaternions, 2);
frames_after_timewarp = size(walk_motions_a(1).warped_bvh, 1);

orig_motions = zeros((quaternion_channels+4)*frames_after_timewarp, motions_in_collection);
% Note the +4 is for 3 root coordinates channels and 1 channel for timewarp info

for i1 = 1 : 35
    orig_motions(:, i1) = reshape( ...
        [ walk_motions_a(i1).warped_bvh(:, 1:3) ...
        [0; log(((walk_motions_a(i1).time_transfer(2:end) - walk_motions_a(i1).time_transfer(1:end-1)))')] ...
        walk_motions_a(i1).warped_quaternions  ], (quaternion_channels+4)*frames_after_timewarp, 1);
end

mean_values = mean(orig_motions');
orig_motions = orig_motions - repmat(mean_values', 1, motions_in_collection);

% Note: calculating too many components will make them all very random
num_of_components = 6;
[ica_scores, mixing_coeff, separating_coeff] = fastica(orig_motions', 'lastEig', num_of_components, 'numOfIC', num_of_components);

ica_scores =  ica_scores';

%% Values that can be changed to explore effects of ICA parameters

index_parameter_to_vary = 1;
offsets_to_parameter = [0.75 0.45 0.15 0 -0.15 -0.45 -0.75];
index_of_starting_motion = 1;

%% Synthesis by linear combination of ICA components

count = 0;
for param_change = offsets_to_parameter
    count = count + 1;
    
    parameters = mixing_coeff(index_of_starting_motion, :)';
    parameters(index_parameter_to_vary) = parameters(index_parameter_to_vary) + param_change;
    
    bvhdata{count} = f_parametric_component_synthesis(ica_scores, parameters, mean_values, walk_motions_a(index_of_starting_motion));
    coordinates{count} = f_fast_bvh_to_coordinates(bvhdata{count}(1:end, :), walk_motions_a(index_of_starting_motion).skel);
end
    
%% Visualization

for i1 = 1:length(coordinates)
    f_animate(bvhdata{i1}, walk_motions_a(1).skel, coordinates{i1}, 'figNum', 101);
    title(['ICA synthesis: offset of ' num2str(offsets_to_parameter(i1)) ' to parameter ' num2str(index_parameter_to_vary)]);
    pause(1.0);
end