clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Description of the example

% This example demonstrates swapping of joint rotations after quaternion
% channels have been decomposed with PCA. Swapping after PCA makes the
% process easier for an end user as the dimension reduction limits the
% number of possible combinations. PCA components seem to combine similarly
% changing joints as some PCA channels affect mainly legs or arms.

% A down side of the PCA based swapping is that the methods seems to have
% all the artifacts that are present when swapping full joints
% (ex_11_joint_rotation_swapping.m) or individual quaternions channels
% (ex_12_quaternion_channel_swapping.m). The visible artifacts include
% broken ground contacts, shakiness and sudden accelerations.

% Main difference between the PCA in this file and the PCA in file
% 'ex_07_pca_synthesis.m' is that here the columns of the input matrix
% are quaternions channels whereas in the other version the columns are
% whole motions.

%%

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

%% Swapping signals after PCA decomposition of quaternions channels

clear channels_to_swap new_motions new_coordinates;

m_ind_1 = 1;
m_ind_2 = 5;

concatenated_quaternions = [walk_motions_a(m_ind_1).warped_quaternions; ...
    walk_motions_a(m_ind_2).warped_quaternions];

[coeffs, ~, latent_synth] = princomp(concatenated_quaternions);
% Note: 'princomp' will be replaced by 'pca' in future Matlab versions
pca_scores = concatenated_quaternions * coeffs;

channels_to_swap{1} = [];
channels_to_swap{2} = 1;
channels_to_swap{3} = 2;
channels_to_swap{4} = 3;
channels_to_swap{5} = 1:4;

%% Visualization

titles = {
    'Original motion'
    '1st PCA channel swapped'
    '2nd PCA channel swapped'
    '3rd PCA channel swapped'
    'PCA channels 1-4 swapped'
    };

for i1 = 1 : length(channels_to_swap)
    m = walk_motions_a(m_ind_1);
    mixed_pca_scores = pca_scores(1:end/2, :);
    mixed_pca_scores(:, channels_to_swap{i1}) = pca_scores((end/2)+1:end, channels_to_swap{i1});
    unmixed_motion = mixed_pca_scores * coeffs';
    m.quaternions = unmixed_motion;
    m.bvhdata = m.warped_bvh;
    time_transfer_inverted = interp1(m.time_transfer, 1:size(m.bvhdata, 1), 1:max(m.time_transfer));
    new_bvhdata{i1} = f_warp_motion(time_transfer_inverted, m);
    new_coordinates{i1} = f_fast_bvh_to_coordinates(new_bvhdata{i1}, walk_motions_a(1).skel);
    f_animate(new_bvhdata{i1}, walk_motions_a(1).skel, new_coordinates{i1}, 'figNum', 100 +i1);
    title(titles{i1}); 
    pause(0.5);
end