clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');
addpath('./external_libraries/FastICA_25');

%% Description of the example

% This example demonstrates swapping of joint rotations after quaternion
% channels have been decomposed with ICA. The resulting motions after
% swapping components seem to have quite random changes compared to same
% procedure with PCA (ex_13_pca_component_swapping.m). However, the
% artifacts seem to be the same as with PCA. An additional problem with ICA
% is that the original motions seem to be badly reproduced if the number of
% components is low, but a high number of components may lead to some of
% them having very random effects.

% Main difference between the ICA in this file and the ICA in file
% 'ex_08_ica_synthesis.m' is that here the columns of the input matrix
% are quaternions channels whereas in the other version the columns are
% whole motions.

% Note that due to the random nature of the ICA, the components are
% different each time they are calculated.

%%

load('matlab_data/walks_a_warped.mat'); % Run script_1 to create this file

%% Swapping signals after ICA decomposition of quaternions channels

clear channels_to_swap new_motions new_coordinates;

m_ind_1 = 1;
m_ind_2 = 5;

concatenated_quaternions = [walk_motions_a(m_ind_1).warped_quaternions; ...
    walk_motions_a(m_ind_2).warped_quaternions];

% Note: calculating too many components will make them all very random
num_of_components = 8;
[ica_scores, mixing_coeff, separating_coeff] = fastica(concatenated_quaternions, ...
    'lastEig', num_of_components, 'numOfIC', num_of_components);

channels_to_swap{1} = [];
channels_to_swap{2} = 1;
channels_to_swap{3} = 2;
channels_to_swap{4} = 3;
channels_to_swap{5} = 1:4;

%% Visualization

titles = {
    'Original motion'
    '1st ICA channel swapped'
    '2nd ICA channel swapped'
    '3rd ICA channel swapped'
    'ICA channels 1-4 swapped'
    };

for i1 = 1 : length(channels_to_swap)
    m = walk_motions_a(m_ind_1);
    mixed_signals = mixing_coeff(1:end/2, :);
    mixed_signals(:, channels_to_swap{i1}) = mixing_coeff((end/2)+1:end, channels_to_swap{i1});
    unmixed_motion = mixed_signals * ica_scores;
    m.quaternions = unmixed_motion;
    m.bvhdata = m.warped_bvh;
    time_transfer_inverted = interp1(m.time_transfer, 1:size(m.bvhdata, 1), 1:max(m.time_transfer));
    new_bvhdata{i1} = f_warp_motion(time_transfer_inverted, m);
    new_coordinates{i1} = f_fast_bvh_to_coordinates(new_bvhdata{i1}, walk_motions_a(1).skel);
    f_animate(new_bvhdata{i1}, walk_motions_a(1).skel, new_coordinates{i1}, 'figNum', 100 +i1);
    title(titles{i1}); 
    pause(0.5);
end