clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%% Notes

% This is an extra analysis showing that motions synthesized with PCA
% synthesis can be reproduced with N-way extrapolation. By default this
% file uses new random PCA parameters on each run. This may result in very
% exaggerated motions.

%% PCA calculation

load('matlab_data/walks_a_warped.mat');

orig_motions = zeros(80*371, 35);

multiplier_for_coordinates = 1;

for i1 = 1 : 35
    orig_motions(:, i1) = reshape( ...
        [ (walk_motions_a(i1).warped_bvh(:, 1:3)*multiplier_for_coordinates) ...
        [0; log(((walk_motions_a(i1).time_transfer(2:end) - walk_motions_a(i1).time_transfer(1:end-1)))')] ...
        walk_motions_a(i1).warped_quaternions  ], 80*371, 1);
end

mean_values = mean(orig_motions');
orig_motions = orig_motions - repmat(mean_values', 1, 35);

[coeffs, ~, latent_synth] = princomp(orig_motions);
% Note: 'princomp' will be replaced by 'pca' in future Matlab versions

% Scores given by 'princomp' have column means substracted with is not
% necessary in our case, thus we recalculate the scores.
pca_scores = orig_motions * coeffs;

%% Percentage of numerical variance contained in the PCA components

% Note: This part shows that the first PCA component contains over 97% of
% the numerical variance. However, this number is somewhat arbitrary as
% using a different unit for the coordinates allows getting pretty much any
% percentage for the first PCA component. It is not well defined what
% is the 'correct' unit for the coordinates as the rotations cannot in any
% case be defined with the same unit. You can try the unit conversion by
% selecting a different value for the 'multiplier_for_coordinates'
% variable.

variance_percents = cumsum(latent_synth)./sum(latent_synth);

figure(1);
plot(variance_percents * 100);
title('Cumulative % of numerical variance contained in the PCA components');

%% Selecting PCA parameters

index_of_starting_motion = 5;
offsets_to_pca_parameters = rand(1, 35) - 0.5; % Can be replaced by manually selected offsets
pca_params = coeffs(index_of_starting_motion, :)' + offsets_to_pca_parameters';

% Note that the last PCA parameter must not be changed to remain inside valid quaternion rotations
pca_params(35) =  coeffs(index_of_starting_motion, 35);

%% Calculation of corresponding N-way extrapolation parameters

extrapolation_parameters = coeffs * pca_params;

%% Synthesis by linear combination of PCA components

bvhdata_pca = f_parametric_component_synthesis(pca_scores, pca_params, mean_values, walk_motions_a(index_of_starting_motion));
bvhdata_pca(:, 1:3) = bvhdata_pca(:, 1:3) * multiplier_for_coordinates;
coordinates_pca = f_fast_bvh_to_coordinates(bvhdata_pca(1:end, :), walk_motions_a(1).skel);

%% Synthesis by N-way extrapolation

bvhdata_extrap = f_extrapolation_nway(extrapolation_parameters, walk_motions_a);
coordinates_extrap = f_fast_bvh_to_coordinates(bvhdata_extrap(1:end, :), walk_motions_a(1).skel);

%% Numerical comparison of results

% Note: Here value 1 would mean total offset of 1 cm summed over all coordinates
% of the joints and all frames of the motions. Basically, the result tells
% that the differences between the PCA and the N-way extrapolation are small
% enough to be explained by rounding errors.

cumulative_difference = sum(sum(abs(coordinates_pca - coordinates_extrap)))

%% Animating resulting motions

f_animate(bvhdata_pca, walk_motions_a(1).skel, coordinates_pca, 'figNum', 101);
title('PCA synthesis');
f_animate(bvhdata_extrap, walk_motions_a(1).skel, coordinates_extrap, 'figNum', 102);
title('N-way extrapolation');
