function [ output_args ] = f_animate(bvhdata, skel, coordinates, varargin)
%F_ANIMATE Animates one motion.
%   F_ANIMATE(BVHDATA,SKELETON,COORDINATES) Animates a motion with
%   default settings and returns handle of the figure.
%
%   F_ANIMATE(BVHDATA,SKELETON,COORDINATES,'PropertyName', PropertyValue)
%   Animates a motion with the given settings.
%   
%   Available settings include:
%
%   'figNum' - Number of figure for displaying animation
%
%   'saveFrames' - A boolean (default false) indicating if produces frames
%   should be saved to folder './video_frames'
%   
%   'fps' - Number of frames per second (default 100)
%
%   'limits' - A array with limits for minimum and maximum for coordinate
%   axes e.g. [ xmin xmax ymin ymax zmin zmax ]
%
%   'lineWidthMult' - A number that acts as multiplier for width of plotted
%   trajectories. This is useful for high quality still images, but it is
%   slower than the default OpenGL trajectories, and works only for the
%   initial point of view.
%
%   'ticks' - A boolean (default true) indicating if ticks should be
%   plotted in the animation.
%
%   'skeletonQuality' - Number indicating the quality of the stick figure
%   model. 1 = low quality (default), 2 = high quality
%   
%   'trajectoryData' - An array of alternative coordinates that are used
%   for plotting trajectories. Useful for having a lower frame rate for
%   animation and higher quality data for the plotted trajectories.
%
%   'color' - An array that sets the rgb color of the skeleton, default is
%   [ 0.0 0.45 0.9 ]
%
%   'backGroundColor' - A string indicating the color (e.g. 'w' or 'b') of
%   the background of the figure.
%
%   'figureSize' - An array indicating the figure size such as [800 600].
%
%   'trajectories' - A boolean (default true) indicating if trajectories
%   should be drawn.
%
%   'smoothTrajectories' - A boolean (default true) indicating if
%   trajectories should be filtered to look smoother.
%
%   'plainTrajectories' - A boolean (default false) indicating if
%   trajectories should only show positions and no velocities.
%
%   'boneRadius' - A number (default 2) indicating the radius of the bones
%   of the stick figure character.
%
%   'view' - An array (default [-45 20]) setting the view rotation.
%
%   'lightPosition' - An array (default [-1000 0 1000]) that sets the
%   position of the light.
%
%   'trajectoryIndices' - A numeric array that has the indices of the
%   body parts for which trajectories will be drawn. The default is to draw
%   trajectories to end for each limb.
%
%   'trajectoryColors' - A numeric array of size [x, 3] that will be used
%   for coloring the trajectories.
%
%   'videoName' - A string such as 'video.avi' that will be used as the
%   file name for the saved video.
%
%   'titleText' - A string that will be the title displayed above the
%   animation.

% Copyright (c) 2015, Klaus Förger

fig_number = 100;
save_frames = false;
fps = 100;
limit_set = false;
limits = [];
line_width_was_set = false;
line_width_mult = 1;
ticks = true;
skeleton_quality = 1;
trajectory_coordinates = [];
backGroundColor = [];
color = [ 0.0 0.45 0.9 ];
figure_size = [];
trajectories = true;
smoothTrajectories = true;
plainTrajectories = false;
bone_radius = 2;
view_rotation = [-45 20];
light_pos = [-1000 0 1000];
trajectoryIndicesSet = false;
trajectoryIndices = [];
colors = [
        0 0 1;
        1 0 0;
        0 0 0;
        1 0 1;
        0 1 1;
        1 0.5 0;
        0.5 0 1;
        1 0.7 0.3;
        0.6 0.2 0;
        1 0 1];
video_name = [];
title_text = [];


if ~isempty(varargin)
    for i1 = 1 : 2 : length(varargin)
        validateattributes(varargin{i1},{'char'}, {});
        switch lower(varargin{i1})
            case {'fignum'}
                validateattributes(varargin{i1+1},{'numeric'},{'size',[1,1]});
                fig_number = varargin{i1+1};
            case 'saveframes'
                validateattributes(varargin{i1+1},{'logical'},{'size',[1,1]});
                save_frames = varargin{i1+1};
            case 'fps'
                validateattributes(varargin{i1+1},{'numeric'},{'size',[1,1]});
                fps = varargin{i1+1};
            case 'limits'
                validateattributes(varargin{i1+1},{'numeric'},{});
                limit_set = true;
                limits = varargin{i1+1};
            case 'linewidthmult'
                validateattributes(varargin{i1+1},{'numeric'},{'size',[1,1]});
                line_width_was_set = true;
                line_width_mult = varargin{i1+1};
             case 'ticks'
                validateattributes(varargin{i1+1},{'logical'},{'size',[1,1]});
                ticks = varargin{i1+1};
            case 'skeletonquality'
                validateattributes(varargin{i1+1},{'numeric'},{'size',[1,1]});
                skeleton_quality = varargin{i1+1};
            case 'trajectorydata'
                validateattributes(varargin{i1+1},{'numeric'},{});
                trajectory_coordinates = varargin{i1+1};
            case 'backgroundcolor'
                validateattributes(varargin{i1+1},{'char'}, {});
                backGroundColor = varargin{i1+1};
            case 'color'
                validateattributes(varargin{i1+1},{'numeric'}, {'size',[1,3]});
                color = varargin{i1+1};
            case 'figuresize'
                validateattributes(varargin{i1+1},{'numeric'},{});
                figure_size = varargin{i1+1};
            case 'trajectories'
                validateattributes(varargin{i1+1},{'logical'},{'size',[1,1]});
                trajectories = varargin{i1+1};
            case 'smoothtrajectories'
                validateattributes(varargin{i1+1},{'logical'},{'size',[1,1]});
                smoothTrajectories = varargin{i1+1};
            case 'plaintrajectories'
                validateattributes(varargin{i1+1},{'logical'},{'size',[1,1]});
                plainTrajectories = varargin{i1+1};
            case 'boneradius'
                validateattributes(varargin{i1+1},{'numeric'},{'size',[1,1]});
                bone_radius = varargin{i1+1};
            case 'view'
                validateattributes(varargin{i1+1},{'numeric'},{'size',[1,2]});
                view_rotation = varargin{i1+1};
             case 'lightposition'
                validateattributes(varargin{i1+1},{'numeric'},{'size',[1,3]});
                light_pos = varargin{i1+1};
            case 'trajectoryindices'
                validateattributes(varargin{i1+1},{'numeric'},{});
                trajectoryIndices = varargin{i1+1};
                trajectoryIndicesSet = true;
            case 'trajectorycolors'
                validateattributes(varargin{i1+1},{'numeric'},{});
                colors = varargin{i1+1};
            case 'videoname'
                validateattributes(varargin{i1+1},{'char'},{});
                video_name = varargin{i1+1};
            case 'titletext'
                validateattributes(varargin{i1+1},{'char'},{});
                title_text = varargin{i1+1};
            otherwise
                disp(['Unknown property: ' varargin{i1}]);
        end
    end
end

if (save_frames)
    close all;
    output_args = figure(fig_number);
    addpath('./external_libraries/ScreenCapture');
else
    output_args = figure(fig_number);
end

if (~isempty(figure_size))
    set(output_args, 'Position', [0 0 figure_size(1) figure_size(2)]);
else
    if (save_frames)
        set(output_args, 'Position', [0 0 800 600]);
    end
end

clf;
set(gcf,'Renderer','opengl');
set(gca,'LooseInset',get(gca,'TightInset'));
view(view_rotation);
axis equal;
grid on;
hold on;

if (~isempty(backGroundColor))
    set(gcf, 'color', backGroundColor);
end

border = bone_radius*2; % Amount of additional empty space, so that the stick figure fits into the space.

if limit_set
    xlim(limits(1:2) + [-border border]);
    ylim(limits(3:4) + [-border border]);
    zlim(limits(5:6) + [-border border]);
else
    [x, y, z] = f_find_xyz_limits(coordinates);
    xlim(x + [-border border]);
    ylim(z + [-border border]);
    zlim(y + [-border border]);
end

if (save_frames)
    set(gca, 'XTickLabel', {});
    set(gca, 'YTickLabel', {});
    set(gca, 'ZTickLabel', {});
end

if (ticks == false)
    set(gca, 'XTick', []);
    set(gca, 'YTick', []);
    set(gca, 'ZTick', []);
end

if (~isempty(video_name))
    video_writer = VideoWriter(video_name);
    video_writer.FrameRate = fps;
    open(video_writer); 
else
    video_writer = [];
end

if (~isempty(title_text))
    title(title_text);
end

last_frame = f_render_skeleton(bvhdata, skel, false, color, save_frames, fps, skeleton_quality, bone_radius, light_pos, video_writer);

% Trajectories
if (trajectories)
    
    if (~isempty(trajectory_coordinates))
        coordinates = trajectory_coordinates;
    end
    
    if (smoothTrajectories)
        vel = filter([1 1 1 1 1]./5, 1, medfilt1(coordinates(1:end-1, :) -  coordinates(2:end, :), 4));
    else
        vel = coordinates(1:end-1, :) -  coordinates(2:end, :);
    end

    if (~trajectoryIndicesSet)
        body_parts = {skel.tree.children};
        end_of_limbs = cellfun(@isempty, body_parts);
        body_parts = 1: length(body_parts);
        body_parts(~end_of_limbs) = [];
        if (length(body_parts) >  6)
            body_parts = body_parts(1:6);
        end
    else
        body_parts = trajectoryIndices;
    end
    
    for i2 = 1 : length(body_parts)
        line_thickness = sqrt(vel(:,(body_parts(i2)*3)-0).^4 + ...
            vel(:,(body_parts(i2)*3)-1).^4 + ...
            vel(:,(body_parts(i2)*3)-2).^4) * 1;
        for i3 = 1 : 1 : size(coordinates, 1) - 2
            
            if (~line_width_was_set)
                if (plainTrajectories)
                    plot3(coordinates(i3:i3+1, (body_parts(i2)*3)-2), ...
                        coordinates(i3:i3+1, (body_parts(i2)*3)-0), ...
                        coordinates(i3:i3+1, (body_parts(i2)*3)-1), ...
                        'Color',  colors(mod(i2 -1, size(colors, 1)) + 1, :));
                else
                    plot3(coordinates(i3:i3+1, (body_parts(i2)*3)-2), ...
                        coordinates(i3:i3+1, (body_parts(i2)*3)-0), ...
                        coordinates(i3:i3+1, (body_parts(i2)*3)-1), ...
                        'Color', colors(mod(i2 -1, size(colors, 1)) + 1, :), ...
                        'LineWidth', (0.5 + (line_thickness(i3) * 0.2)));
                end
            else
                line_width_t = ((0.9) + (line_thickness(i3) * 0.2));
                line_width_n = ((0.9) + (line_thickness(i3+1) * 0.2));
                
                a = coordinates(i3:i3+2, (body_parts(i2)*3)-2);
                b = coordinates(i3:i3+2, (body_parts(i2)*3)-0);
                c = coordinates(i3:i3+2, (body_parts(i2)*3)-1);
                
                v1 = camtarget - campos;
                v1 = v1 / norm(v1);
                v2 = [a(1) b(1) c(1)] - [a(2) b(2) c(2)];
                v2 = v2 / norm(v2);
                v3 = [a(2) b(2) c(2)] - [a(3) b(3) c(3)];
                v3 = v3 / norm(v3);
                
                mt = cross(v1, v2) * line_width_t * line_width_mult;
                mn = cross(v1, v3) * line_width_n * line_width_mult;
                
                x = [a(1) a(1) a(2) a(2)] + ([-1*mt(1) 1*mt(1) 1*mn(1) -1*mn(1)]);
                y = [b(1) b(1) b(2) b(2)] + ([-1*mt(2) 1*mt(2) 1*mn(2) -1*mn(2)]);
                z = [c(1) c(1) c(2) c(2)] + ([-1*mt(3) 1*mt(3) 1*mn(3) -1*mn(3)]);
                
                if (norm([x(2) y(2) z(2)] - [x(3) y(3) z(3)]) + norm([x(1) y(1) z(1)] - [x(4) y(4) z(4)]) > norm([x(2) y(2) z(2)] - [x(4) y(4) z(4)])  + norm([x(1) y(1) z(1)] - [x(3) y(3) z(3)]))
                    x([3 4]) = x([4 3]);
                    y([3 4]) = y([4 3]);
                    z([3 4]) = z([4 3]);
                end
                
                fill3(x, y, z, colors(mod(i2 -1, size(colors, 1)) + 1, :), 'EdgeColor' , 'none', 'FaceLighting','none');
            end
        end
    end
end

drawnow;

actual_figure_pos = get(output_args, 'Position');

if ( (save_frames || ~isempty(video_writer)) && trajectories)
    for i1 = 1 : 50
        if (save_frames)
            screencapture(0, actual_figure_pos, 'target', ['video_frames/frame_' sprintf('%05d',  last_frame+i1) '.png']);
        end
        if (~isempty(video_writer))
            frame = getframe(gcf);
            writeVideo(video_writer, frame);
        end
    end
    close(output_args);
end

if (~isempty(video_writer))
    close(video_writer); 
else

end

