function [hg_transform] = f_create_rotation_bone(coordsB, radius, num_segments, surf_color)
%F_CREATE_ROTATION_BONE Creates a bone that is show in an animation.
%
%   F_CREATE_ROTATION_BONE(COORDSB,R,N,SURF_COLOR,LINES)
%   COORDSB is an array holding the coordinate offset of the bone
%   RADIUS is the radius of the cylinder
%   NUM_SEGMENTS is the number of pieces in the cylinder and the ball joint
%   SURF_COLOR is a color definition such as [0.53 0.10 0.95]
%   Returns hgtransform for the bone.

% Copyright (c) 2015, Klaus Förger

coordsA = [0 0 0];
coordsB = coordsB([1 3 2]);

% Calculating the length of the cylinder
length_cyl = norm(coordsB - coordsA);

% Creating a circle in the YZ plane
t = linspace(0, 2*pi, num_segments)';
x2 = radius * cos(t);
x3 = radius * sin(t);

% Data for a circle to cylinder caps
x1 = [0 length_cyl];
xx1 = repmat(x1, length(x2), 1);
xx2 = repmat(x2, 1, 2);
xx3 = repmat(x3, 1, 2);

% Drawing two filled circles to close the cylinder
hold on
[sx, sy, sz] = sphere(num_segments);
clear colors;
colors(1:size(sz, 1), 1:size(sz, 2), [1 2 3]) = 1;
colors(1:size(sz, 1), 1:size(sz, 2), 1) = surf_color(1);
colors(1:size(sz, 1), 1:size(sz, 2), 2) = surf_color(2);
colors(1:size(sz, 1), 1:size(sz, 2), 3) = surf_color(3);
m = 2;

Ball = surf((sx * radius * m) + length_cyl, (sy * radius * m), sz * radius * m, colors);


% Plotting the cylinder along the X-Direction with required length starting
% from Origin
clear colors;
colors(1:size(xx3, 1), 1:size(xx3, 2), [1 2 3]) = 1;
colors(1:size(xx3, 1), 1:size(xx3, 2), 1) = surf_color(1);
colors(1:size(xx3, 1), 1:size(xx3, 2), 2) = surf_color(2);
colors(1:size(xx3, 1), 1:size(xx3, 2), 3) = surf_color(3);
Cylinder=surf(xx1,xx2,xx3, colors);

hg_transform = hgtransform;
set(Ball,'Parent',hg_transform);
set(Cylinder,'Parent',hg_transform);

% Calculating transform for the objects
mat = f_return_transformation_matrix([0.001 0.001 0.001],coordsB);
set(hg_transform,'Matrix', mat);

% Setting the color to the cylinder and the caps
set(Cylinder,'FaceColor',surf_color)
set(Ball,'FaceColor',surf_color)

% Removing edge lines from polygons
set(Cylinder,'EdgeColor','none','LineStyle','none')
set(Ball,'EdgeColor','none','LineStyle','none')

% Shading
set(Cylinder,'FaceLighting','phong','FaceColor','flat');
set(Ball,    'FaceLighting','phong','FaceColor','flat');
set(Cylinder, 'LineSmoothing','on');
set(Ball, 'LineSmoothing','on');
material([0.0 0.5 0.15]*(2/3));



%%

function [matrix] = f_return_transformation_matrix(X1,X2)
%F_RETURN_TRANSFORMATION_MATRIX(X1,X2) Returns a transformation matrix
%   X1 and X2 are the 3x1 vectors of the two points

% Defining Unit vector along the X-direction
unit_Vx=[1 0 0];

% Calculating the angle between the x direction and the required direction
% of cylinder through dot product
angle_X1X2=acos( dot( unit_Vx,(X2-X1) )/( norm(unit_Vx)*norm(X2-X1)) );

% Finding the axis of rotation (single rotation) to roate the cylinder in
% X-direction to the required arbitrary direction through cross product
axis_rot=cross([1 0 0],(X2-X1));

Ry = makehgtform('axisrotate',axis_rot,angle_X1X2);

T = makehgtform('translate',X1);

matrix = T*Ry;
