function [ final_bvhdata ] = f_extrapolation_nway( parameters, motions )
%F_EXTRAPOLATION_NWAY N-way extrapolation between motions
%
%   F_EXTRAPOLATION_NWAY(PARAMETERS,MOTIONS)
%   PARAMETERS are the weights of each input motion. They must sum to one.
%   MOTIONS is a prewarped motion collection.
%   Returns BVH data for the new motion.

% Copyright (c) 2015, Klaus Förger

bvh_size = size(motions(1).bvhdata, 2);
rot_dof = (bvh_size/3) - 1;

if (sum(parameters) < 0.999999 || sum(parameters) > 1.000001)
    disp('The parameters must sum to one to represent a valid interpolation or extrapolation.');
end

motion_count = length(motions);

% Motion interpolation according to parameters

quat = zeros(size(motions(1).warped_quaternions, 1), size(motions(1).warped_quaternions, 2), motion_count);
xyz = zeros(size(motions(1).warped_offset_root, 1), size(motions(1).warped_offset_root, 2), motion_count);
time_transfer = zeros(size(motions(1).time_transfer, 1), size(motions(1).time_transfer, 2), 8);

for i1 = 1 : motion_count
    quat(:, :, i1) = motions(i1).warped_quaternion_offsets .* parameters(i1);
    xyz(:, :, i1) = motions(i1).warped_offset_root .* parameters(i1);
    time_transfer(:, :, i1) = [0; log(((motions(i1).time_transfer(2:end) - motions(i1).time_transfer(1:end-1)))')] .* parameters(i1);
end

quat_offset = sum(quat, 3);
xyz_offset = sum(xyz, 3);
time_transfer_offset = sum(time_transfer, 3);

quat_new = zeros(size(quat_offset));

for i1 = 1 : 4 : rot_dof*4
    quat_new(1 : size(quat_offset, 1), i1:i1+3) = qmult(qconj(quat_offset(1 : size(quat_offset, 1), i1:i1+3)), motions(1).quaternions(1 : size(quat_offset, 1), i1:i1+3));
end

bvh_new = zeros(size(motions(1).warped_bvh));

for i2 = 1 : rot_dof
    bvh_new(:, (i2-1)*3+4:(i2-1)*3+6) = f_SpinCalc('QtoEA312', quat_new(:, (i2-1)*4+1:(i2-1)*4+4), 0.0001, 0);
end

bvh_new(:, 1:3) = motions(1).bvhdata(1:size(xyz_offset, 1), 1:3) - xyz_offset;

clear motion;
motion(1).skel = motions(1).skel;
motion(1).bvhdata = bvh_new;

motion(1).quaternions = f_calculate_quaternions(bvh_new);

trans_temp = cumsum(exp(time_transfer_offset))';

time_tranfer = interp1(trans_temp, 1:size(motion(1).bvhdata, 1), 1:max(trans_temp));
time_tranfer(1) = 1;

final_bvhdata = f_warp_motion(time_tranfer, motion(1));


end

