function [ final_bvhdata ] = f_interpolation_nway( parameters, motions, post_processing_enabled )
%F_INTERPOLATION_NWAY N-way interpolation between motions
%
%   F_INTERPOLATION_NWAY(PARAMETERS,MOTIONS)
%   PARAMETERS are the weights of each input motion. They must sum to one.
%   MOTIONS is a prewarped motion collection.
%   Returns BVH data for the new motion.
%
%   F_INTERPOLATION_NWAY(PARAMETERS,MOTIONS,POST_PROCESSING_ENABLED)
%   Giving 'POST_PROCESSING_ENABLED = false' leaves some acceleration
%   artifacts unfixed. The default value is true.

% Copyright (c) 2015, Klaus Förger

bvh_size = size(motions(1).bvhdata, 2);
rot_dof = (bvh_size/3) - 1;

if (sum(parameters) == 0)
    error('Zero parameters given.')
end

if nargin < 3
    post_processing_enabled = true;
end

motion_count = length(motions);

parameters = parameters ./ sum(parameters); % this restricts the results to interpolation

% Motion interpolation according to parameters

quat = zeros(size(motions(1).warped_quaternions, 1), size(motions(1).warped_quaternions, 2), motion_count);
xyz = zeros(size(motions(1).warped_offset_root, 1), size(motions(1).warped_offset_root, 2), motion_count);
time_transfer = zeros(size(motions(1).time_transfer, 1), size(motions(1).time_transfer, 2), 8);
time_this = zeros(size(motions(1).time_this, 1), size(motions(1).time_this, 2), motion_count);
warped_velocity = zeros(size(motions(1).warped_velocity, 1), size(motions(1).warped_velocity, 2), 8);


for i1 = 1 : motion_count
    quat(:, :, i1) = motions(i1).warped_quaternion_offsets .* parameters(i1);
    xyz(:, :, i1) = motions(i1).warped_offset_root .* parameters(i1);
    time_transfer(:, :, i1) = motions(i1).time_transfer .* parameters(i1);
    time_this(:, :, i1) = motions(i1).time_this .* parameters(i1);
    warped_velocity(:, :, i1) = motions(i1).warped_velocity .* parameters(i1);
end

quat_offset = sum(quat, 3);
xyz_offset = sum(xyz, 3);
time_transfer_offset = sum(time_transfer, 3);
time_offset = sum(time_this, 3); % Note: time_offset and time_transfer_offset contain the same information
warped_velocity_mean = sum(warped_velocity, 3);

quat_new = zeros(size(quat_offset));

for i1 = 1 : 4 : rot_dof*4
    quat_new(1 : size(quat_offset, 1), i1:i1+3) = qmult(qconj(quat_offset(1 : size(quat_offset, 1), i1:i1+3)), motions(1).quaternions(1 : size(quat_offset, 1), i1:i1+3));
end

bvh_new = zeros(size(motions(1).warped_bvh));

for i2 = 1 : rot_dof
    bvh_new(:, (i2-1)*3+4:(i2-1)*3+6) = f_SpinCalc('QtoEA312', quat_new(:, (i2-1)*4+1:(i2-1)*4+4), 0.0001, 0);
end

bvh_new(:, 1:3) = motions(1).bvhdata(1:size(xyz_offset, 1), 1:3) - xyz_offset;

clear motion;
motion(1).skel = motions(1).skel;
motion(1).bvhdata = bvh_new;

motion(1).quaternions = f_calculate_quaternions(bvh_new);

time_transfer_inverted = interp1(time_transfer_offset, 1:size(bvh_new, 1), 1:max(time_transfer_offset));
time_transfer_inverted(1) = 1;

% Note: this can end up with sudden accelerations near 50%-50% interpolations,
% which is now fixed with velocity based post-processing
semi_final_bvhdata = f_warp_motion(time_transfer_inverted, motion(1));

if (post_processing_enabled)
    xyz_new = f_fast_bvh_to_coordinates(semi_final_bvhdata, motions(1).skel);

    velocity_semi_final = f_overall_velocity_for_motion(xyz_new);

    velocity_desired = interp1(1:length(warped_velocity_mean), warped_velocity_mean, time_transfer_inverted);

    % This is a post-processing step that fixes sudden accelerations caused by time warping
    fixed_time = ones(size(time_offset));
    for i1 = 2 : length(time_offset)
        frames = ceil(time_offset(i1 -1)) : floor(time_offset(i1));
        fixed_time(i1) = fixed_time(i1 - 1) + ((time_offset(i1) - time_offset(i1 - 1)) * ( mean(velocity_semi_final(frames)) / mean(velocity_desired(frames))));
    end
    fixed_time(2:end) = fixed_time(2:end) * (((time_offset(end)*0.5) + (fixed_time(end)*0.5) )/ fixed_time(end));

    time_transfer_final = interp1(motions(1).time_target, fixed_time(:), 1:motions(1).time_target(end), 'linear');

    time_transfer_final_inverted = interp1(time_transfer_final, 1:size(bvh_new, 1), 1:max(time_transfer_final));

    final_bvhdata = f_warp_motion(time_transfer_final_inverted, motion(1));
else
    final_bvhdata = semi_final_bvhdata;
end


end

