function [ final_bvhdata ] = f_interpolation_pairwise( m_from, m_to, percent, time_transfer )
%F_INTERPOLATION_PAIRWISE Pairwise interpolation between motions
%
%   F_INTERPOLATION_PAIRWISE(M_FROM,M_TO,PERCENT)
%   M_FROM and M_TO are the two motions.
%   PERCENT tells the extrapolation amount. Value must be between 0 and 1.
%   Returns BVH data for the new motion.
%   
%   F_INTERPOLATION_PAIRWISE(M_FROM,M_TO,PERCENT,TIME_TRANSFER)
%   TIME_TRANSFER allows specifying a custom time warping between the
%   motions. The default is to warp the motions to the same duration, and
%   not to do any fine tuning.

% Copyright (c) 2015, Klaus Förger

bvh_size = size(m_from.bvhdata, 2);
rot_dof = (bvh_size/3) - 1;

m_new = [];

% To be on the safe side:
m_from.quaternions = f_fix_quaternions(m_from.quaternions);
m_to.quaternions = f_fix_quaternions(m_to.quaternions);

% Warping second motion to same length as the first one.
if (nargin < 4)
    time_transfer = 1:(size(m_to.bvhdata, 1) - 1) / (size(m_from.bvhdata, 1)-1):size(m_to.bvhdata, 1);
end
m_to_bvh_warped = f_warp_motion(time_transfer, m_to);
m_to_quat_warped = f_calculate_quaternions(m_to_bvh_warped);

% Interpolation

m_new.quaternions = zeros(size(m_from.quaternions));

for i1 = 1 : rot_dof
    for i2 = 1 : size(m_new.quaternions, 1)
        s = 1+(4*(i1-1)) : 4+(4*(i1-1));
        m_new.quaternions(i2, s) = f_slerp(m_from.quaternions(i2, s), m_to_quat_warped(i2, s), percent, 0.001);
    end
end

m_new.quaternions = f_fix_quaternions(m_new.quaternions);

m_new.bvhdata = zeros(size(m_from.bvhdata));
m_new.bvhdata(:, 1:3) = (m_from.bvhdata(:, 1:3) * (1-percent)) +  (m_to_bvh_warped(:, 1:3) * percent);

% Time warping the resulting motion to the duration the reflects the
% interpolation parameter.
target_length = round((size(m_from.bvhdata, 1) * (1-percent) + (size(m_to.bvhdata, 1) * percent)));
time_transfer_back = 1:(size(m_from.bvhdata, 1) - 1) / (target_length-1):size(m_from.bvhdata, 1);

final_bvhdata = f_warp_motion(time_transfer_back, m_new);


end

