function [ output_motion ] = f_multiresolution_editing( input_signals, multipliers )
%F_MULTIRESOLUTION_EDITING Multiplies frequency bands
%   F_MULTIRESOLUTION_EDITING(INPUT_SIGNALS,MULTIPLIERS)
%   This function splits the input motion signals to multiple resolution
%   bands with a wavelet transform, multiplies the bands, and sums them back
%   together. The multipliers are given as an array of seven numbers. The
%   signals must have at least 256 frames.
%   Returns the edited signals as an array.
%
%   The original algorithm is presented in: Bruderlin, A., Williams, L.: 
%   Motion signal processing. In: Proc. of the 22nd annual conference on 
%   Computer graphics and interactive techniques. pp. 97–104. ACM (1995)

% Copyright (c) 2015, Klaus Förger

multipliers(8) = 1;

[max_frames, dimensions] = size(input_signals);

%% Extending the signals from begining and end with a constant value

ext_1 = 400;
ext_2 = 401;
ext_3 = 800;

input_signals_ext = zeros(ext_3 + max_frames, dimensions);

input_signals_ext(1:ext_2, :) = repmat(input_signals(2, :), ext_2, 1);
input_signals_ext(ext_2:max_frames+ext_1, :) = input_signals(:, :);
input_signals_ext(ext_2 + max_frames  : ext_3 + max_frames, :) = repmat(input_signals(max_frames, :), ext_3 - ext_1, 1);


%% This calculates how many bands can be produced

bands = 8; % Requires 256 samples ~ 2.56 seconds of motion @ 100 fps

%% Filtering with the wavelet

% Wavelet definition: cbabc
a = 3/8;
b = 1/4;
c = 1/16;

signals = zeros(ext_3 + max_frames, dimensions, bands);
signals(:, :, 1) = input_signals_ext;

for band = 1 : bands - 1
    wavelet = [c zeros(1,(2^(band-1))-1) b zeros(1,(2^(band-1))-1) a zeros(1,(2^(band-1))-1) b zeros(1,(2^(band-1))-1) c];
    signals(:, :, band+1) = filter(wavelet, 1, signals(:, :, band));
    signals(1 : ext_3 + max_frames - (2 * (2^(band - 1))), :, band+1) = signals(1 + (2 * (2^(band - 1))) : ext_3 + max_frames, :, band+1);
end

%% Separating the bands

temp_result = zeros(max_frames, size(signals, 2), size(signals, 3));
for band = 1 : bands - 1
    signals(:, :, band) = signals(:, :, band) - signals(:, :, band + 1);
    temp_result(:, :, band) = signals(ext_2 : max_frames + ext_1, :, band);
end

temp_result(:, :, bands) = signals(ext_2 : max_frames + ext_1, :, bands);

%% Multiplying and joining the bands

output_motion = zeros(size(input_signals));

for i1 = 1 : 8
    temp_result(:, :, i1) = temp_result(:, :, i1) * multipliers(i1);
end

for i1 = 1 : dimensions
    output_motion(:, i1) = sum(temp_result(:, i1, :), 3);
end

end
