function [ shown_frames_count ] = f_render_skeleton( bvhdata, skel, clear_in_the_end, color, save_frames, fps, skeleton_quality, bone_radius, light_pos, video_writer)
%F_RENDER_SKELETON A function used in the animation loop

% Copyright (c) 2015, Klaus Förger

if nargin < 5
    save_frames = false;
end

if nargin < 6
    fps = 100;
end

if nargin < 7
    skeleton_quality = 1;
end

if nargin < 8
    bone_radius = 2;
end

if nargin < 9
    light_pos = [-1000 0 1000];
end

if nargin < 10
    video_writer = [];
end

handle = [];

time_end = size(bvhdata, 1) / fps;

light_count = 0;

% Creating bones and transformation hierarchy
for i1 = 1 : length(skel.tree) 
    new_transform = hgtransform();
    
    if (skel.tree(i1).parent > 0)
        
        if (isempty(skel.tree(i1).children) && light_count < 8)
            % Matlab seems want the light to be set for each end of limb
            light_count = light_count + 1;
            light('Position', light_pos, 'Style', 'infinite');
        end
        
        if (skeleton_quality > 1 || save_frames)
            % This enables higher quality for videos
            bone_visuals = f_create_rotation_bone(skel.tree(i1).offset, bone_radius, 12, color);
        else
            bone_visuals = f_create_rotation_bone(skel.tree(i1).offset, bone_radius, 6, color);
        end
        
        set(bone_visuals, 'Parent', handle(skel.tree(i1).parent));
        set(new_transform, 'Parent', handle(skel.tree(i1).parent));
    end

    handle(end+1) = new_transform;
end



% Precalculating hg transforms
disp('Precalculating visualization...');
ident = [1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 1];
hg1 = zeros(4, 4, size(bvhdata, 1), length(handle));
for i1 = 1 : size(bvhdata, 1)
    for i2 = 1 : length(handle)

        % Calculating rotation matrices
        if (isempty(skel.tree(i2).rotInd))
            hg1(:, :, i1, i2) = ident;
        else
            R1 = ident;
            R1(1, 1) = cos(-bvhdata(i1, skel.tree(i2).rotInd(3)) * (pi/180));
            R1(3, 3) = R1(1, 1);
            R1(1, 3) = sin(-bvhdata(i1, skel.tree(i2).rotInd(3)) * (pi/180));
            R1(3, 1) = -R1(1, 3);
            
            R2 = ident;
            R2(2, 2) = cos(-bvhdata(i1, skel.tree(i2).rotInd(1)) * (pi/180));
            R2(3, 3) = R2(2, 2);
            R2(2, 3) = -sin(-bvhdata(i1, skel.tree(i2).rotInd(1)) * (pi/180));
            R2(3, 2) = -R2(2, 3);
            
            R3 = ident;
            R3(1, 1) = cos(-bvhdata(i1, skel.tree(i2).rotInd(2)) * (pi/180));
            R3(2, 2) = R3(1, 1);
            R3(1, 2) = -sin(-bvhdata(i1, skel.tree(i2).rotInd(2)) * (pi/180));
            R3(2, 1) = -R3(1, 2);
            
            % Note: These command give the same results, but are about ten times slower
            %R1 = makehgtform('yrotate',-bvhdata(i1, skel.tree(i2).rotInd(3)) * (pi/180) );
            %R2 = makehgtform('xrotate',-bvhdata(i1, skel.tree(i2).rotInd(1)) * (pi/180) );
            %R3 = makehgtform('zrotate',-bvhdata(i1, skel.tree(i2).rotInd(2)) * (pi/180) );
            hg1(:, :, i1, i2) = R1 * R2 * R3;
        end
        

        % Adding translation to the matrix
        if (isempty(skel.tree(i2).posInd))
            hg1(1:3, 4, i1, i2) = skel.tree(i2).offset([1 3 2]);
        else
            hg1(1:3, 4, i1, i2) = bvhdata(i1,skel.tree(i2).posInd([1 3 2]));
        end
        
    end
end
disp('Done.')

shown_frames_count = 0;

if (save_frames || ~isempty(video_writer))
    for frame_number = 1 : size(bvhdata, 1)
        shown_frames_count = shown_frames_count + 1;
        
        % Updating the hg transformations
        for i2 = 1 : length(handle)
            set(handle(i2),'Matrix',hg1(:, :, frame_number, i2));
        end

        drawnow;

        if (shown_frames_count == 1)
            pause(0.5); % This is usually enought to avoid broken first frames
            actual_figure_pos = get(gcf, 'Position');
        end
        if (~isempty(video_writer))
            frame = getframe(gcf);
            writeVideo(video_writer, frame);
        end
        if (save_frames)
            screencapture(0, actual_figure_pos, 'target', ['video_frames/frame_' sprintf('%05d',  shown_frames_count) '.png']);
        end
    end
    shown_frames_count = frame_number;
    
else % Realtime animation, not saving frames to disk

    tic;
    prev_frame = 0;
    frame_number = 1;
    last_frame_not_shown = true;
    
    while (last_frame_not_shown)
        
        if (frame_number == size(bvhdata, 1))
            last_frame_not_shown = false;
        end
        
        if (frame_number > prev_frame)
            
            
            % Updating the hg transformations
            for i2 = 1 : length(handle)
                set(handle(i2),'Matrix',hg1(:, :, frame_number, i2));
            end
            
            shown_frames_count = shown_frames_count + 1;
            drawnow;
        end
        
        time_next = (frame_number + 1) * (1/fps);
        
        time_now = toc;
        if (time_now < time_next)
            pause(min((time_next - time_now), 0.001));
        end
        
        prev_frame = frame_number;
        time_now = toc;
        frame_number = min(ceil(time_now * fps), size(bvhdata, 1));

    end
end

if (clear_in_the_end)
    while (~isempty(handle))
        delete(handle(end));
        handle(end) = [];
    end
end

end

