function [ bvh_new ] = f_warp_motion( timing, motion )
%F_WARP_MOTION(TIMING,MOTION) Performs timewarping with the given timing
%
%   TIMING is an array of the same length as the new motion. The values
%   in the array tell from which original frame the new frames are created
%   from. For example, TIMING = [1 0.5 1 1.5 2 2.5 3] would mean creating a
%   new motion with seven frames and slowing down the original motion by 
%   50 percent.
%
%   MOTION is the motion struct containing the original motion. The struct
%   must have bvhdata with valid root coordinates, and quaternion rotations
%   for the joints.
%
%   Returns BVH data of the warped motion.

% Copyright (c) 2015, Klaus Förger

% Note: Performing this operation on joint angles instead of quaternions
% would break when gimbal lock is near.

bvh_size = size(motion.bvhdata, 2);
rot_dof = (bvh_size/3) - 1;

% Time warp of rotations
quat_new1 = zeros(length(timing), rot_dof*4);
for i1 = 1 : rot_dof*4
    quat_new1(:, i1) = interp1(1:size(motion.bvhdata, 1), motion.quaternions(:, i1), timing);
end


% Time warp of root position
pos_new1 = zeros(length(timing), 3);
for i1 = 1 : 3
    pos_new1(:, i1) = interp1(1:size(motion.bvhdata, 1), motion.bvhdata(:, i1), timing);
end

% Converting everything to BVH -format
bvh_new = zeros(length(timing), bvh_size);
for i2 = 1 : rot_dof
    bvh_new(:, (i2-1)*3+4:(i2-1)*3+6) = f_SpinCalc('QtoEA312', quat_new1(:, (i2-1)*4+1:(i2-1)*4+4), 0.0001, 0);
end

bvh_new(:, 1:3) = pos_new1;

end

