clear;

addpath('./function_library');
addpath('./external_libraries/GPmat/matlab');
addpath('./external_libraries/SpinCalc');
addpath('./external_libraries/quaternions');

%%

% This script loads a collection of 35 walking motions, finds how steps in
% the motions match and time warps all the motions to have same timing as
% the first motion in the collection. The collection is then saved to disk
% to be readily used by other examples.

%% Load originals

files = {
    'motion_data/walk_a_1.bvh'
    'motion_data/walk_a_2.bvh'
    'motion_data/walk_a_3.bvh'
    'motion_data/walk_a_4.bvh'
    'motion_data/walk_a_5.bvh'
    'motion_data/walk_a_6.bvh'
    'motion_data/walk_a_7.bvh'
    'motion_data/walk_a_8.bvh'
    'motion_data/walk_a_9.bvh'
    'motion_data/walk_a_10.bvh'
    'motion_data/walk_a_11.bvh'
    'motion_data/walk_a_12.bvh'
    'motion_data/walk_a_13.bvh'
    'motion_data/walk_a_14.bvh'
    'motion_data/walk_a_15.bvh'
    'motion_data/walk_a_16.bvh'
    'motion_data/walk_a_17.bvh'
    'motion_data/walk_a_18.bvh'
    'motion_data/walk_a_19.bvh'
    'motion_data/walk_a_20.bvh'
    'motion_data/walk_a_21.bvh'
    'motion_data/walk_a_22.bvh'
    'motion_data/walk_a_23.bvh'
    'motion_data/walk_a_24.bvh'
    'motion_data/walk_a_25.bvh'
    'motion_data/walk_a_26.bvh'
    'motion_data/walk_a_27.bvh'
    'motion_data/walk_a_28.bvh'
    'motion_data/walk_a_29.bvh'
    'motion_data/walk_a_30.bvh'
    'motion_data/walk_a_31.bvh'
    'motion_data/walk_a_32.bvh'
    'motion_data/walk_a_33.bvh'
    'motion_data/walk_a_34.bvh'
    'motion_data/walk_a_35.bvh'
    };

for i = 1 : length(files)
    disp(['loading file ' int2str(i)]);
    [skeleton, channels, framelength] = bvhReadFile([files{i}]);
    walk_motions_a(i).skel =  skeleton;
    walk_motions_a(i).bvhdata =  channels;
    walk_motions_a(i).framelength =  framelength;
    walk_motions_a(i).filename = files{i};
    walk_motions_a(i).quaternions = f_calculate_quaternions(channels);
    
    % Here are two versions of coordinate calculation. The first is slow and
    % general. The second is faster, but more picky about the skeleton structure.
    
    %walk_motions_a(i).coordinates = f_bvh_to_coordinates(channels, skeleton); 
    walk_motions_a(i).coordinates = f_fast_bvh_to_coordinates(channels, skeleton);
end

%% Timing all to the walks to the first motion

step_detection_threshold = 0.15;
channel_l = 2+(3*18); % Index of the left foot 
channel_r = 2+(3*22); % Index of the right foot

temp_data = [];

% This code is not very pretty and it generalizes poorly to other motion
% collections. It would be better with proper implementation of dynamic
% time warping.
for i1 = 1 : length(walk_motions_a)

    velocity = f_overall_velocity_for_motion(walk_motions_a(i1).coordinates);
    walk_motions_a(i1).velocity = velocity;
    
    temp_data(i1).median_size = round( (15 * (1 / mean(velocity)) - 1) / 2 ) * 2 + 1;
    
    motion_l = tsmovavg(walk_motions_a(i1).coordinates(:, channel_l-1 : channel_l+1)', 's', 8);
    motion_r = tsmovavg(walk_motions_a(i1).coordinates(:, channel_r-1 : channel_r+1)', 's', 8);
    speed_l = abs(sum(motion_l(:, 2:length(motion_l)) - motion_l(:, 1:length(motion_l)-1)));
    speed_r = abs(sum(motion_r(:, 2:length(motion_r)) - motion_r(:, 1:length(motion_r)-1)));
    speed_l(isnan(speed_l)) = 0;
    speed_r(isnan(speed_r)) = 0;
    
    temp_data(i1).speed_l = speed_l;
    temp_data(i1).speed_r = speed_r;
    
    steps_l = speed_l - (max(speed_l) * step_detection_threshold);
    steps_l(steps_l(:) < 0 ) = 0;
    steps_l(steps_l(:) > 0 ) = 1;
    steps_l = medfilt1(steps_l, temp_data(i1).median_size);
    
    steps_r = speed_r - (max(speed_r) * step_detection_threshold);
    steps_r(steps_r(:) < 0 ) = 0;
    steps_r(steps_r(:) > 0 ) = 1;
    steps_r = medfilt1(steps_r, temp_data(i1).median_size);
    
    temp_data(i1).min_max_l = steps_l(2:length(steps_l)) - steps_l(1:length(steps_l)-1);
    temp_data(i1).min_max_r = steps_r(2:length(steps_r)) - steps_r(1:length(steps_r)-1);
end

for i1 = 1 : length(walk_motions_a)
    walk_motions_a(i1).time_target = [];
    walk_motions_a(i1).time_this = [];
    walk_motions_a(i1).time_transfer = [];
    walk_motions_a(i1).warped_bvh = [];
    walk_motions_a(i1).warped_velocity = [];
    walk_motions_a(i1).warped_offset_root = [];
    walk_motions_a(i1).warped_quaternions = [];
    walk_motions_a(i1).warped_quaternion_offsets = [];
end

walk1 =  temp_data(1);
walk1.bvhdata = walk_motions_a(1).bvhdata;
walk1.quaternions = walk_motions_a(1).quaternions;

for i1 = 1 : length(walk_motions_a)
    
    % Note: This code uses only data from the left foot to ensure that
    % the segmentation based on the steps is compatible within all 35 motions.
    [ time_target time_this ] = f_connect_locomotion_timings( ...
        walk1(1).min_max_l, walk1(1).min_max_l, ...
        temp_data(i1).min_max_l, temp_data(i1).min_max_l );
    
    walk_motions_a(i1).time_target = time_target;
    walk_motions_a(i1).time_this = time_this;
    
    walk_motions_a(i1).time_transfer = interp1(time_target(:), time_this(:), 1:time_target(end), 'linear');
    
    % Note: this can end up with sudden accelerations that need to be post-processed away
    walk_motions_a(i1).warped_bvh = f_warp_motion(walk_motions_a(i1).time_transfer, walk_motions_a(i1));
    walk_motions_a(i1).warped_velocity = interp1(1:size(walk_motions_a(i1).velocity, 1), walk_motions_a(i1).velocity, walk_motions_a(i1).time_transfer);
    walk_motions_a(i1).warped_offset_root = walk1.bvhdata(1:size(walk_motions_a(i1).warped_bvh, 1), 1:3) - walk_motions_a(i1).warped_bvh(:, 1:3); 
    walk_motions_a(i1).warped_quaternions = f_calculate_quaternions(walk_motions_a(i1).warped_bvh);
    walk_motions_a(i1).warped_quaternion_offsets = zeros(size(walk_motions_a(i1).warped_quaternions));
    
    for i2 = 1 : 4 : 76
        for i3 = 1 : size(walk_motions_a(i1).warped_quaternions, 1)
            walk_motions_a(i1).warped_quaternion_offsets(i3, i2:i2+3) = qmult(walk1(1).quaternions(i3, i2:i2+3), qconj(walk_motions_a(i1).warped_quaternions(i3, i2:i2+3)));
        end
    end
end

%% Save data

save('matlab_data/walks_a_warped.mat', 'walk_motions_a');
disp('Saved data to disk.');

%% Animating one example motion

f_animate(walk_motions_a(5).bvhdata, walk_motions_a(5).skel, walk_motions_a(5).coordinates, 'figNum', 100);
